from . import _api, _fipiran, _DataFrame, _read_html, _to_datetime


class Fund:
    __slots__ = 'reg_no'

    def __init__(self, reg_no: int | str):
        self.reg_no = reg_no

    def __repr__(self):
        return f'{type(self).__name__}({self.reg_no!r})'

    def asset_allocation(self) -> dict:
        """Return a dict where values are percentage of each kind of asset."""
        return _api(f'chart/getfundchartasset?regno={self.reg_no}')

    def issue_cancel_history(self) -> _DataFrame:
        j = _api(f'chart/getfundchart?regno={self.reg_no}')
        df = _DataFrame(j, copy=False)
        df['date'] = _to_datetime(df['date'])
        df.set_index('date', inplace=True)
        return df

    def nav_history(self) -> _DataFrame:
        j = _api(f'chart/getfundnetassetchart?regno={self.reg_no}')
        df = _DataFrame(j, copy=False)
        df['date'] = _to_datetime(df['date'])
        df.set_index('date', inplace=True)
        return df

    def info(self):
        return _api(f'fund/getfund?regno={self.reg_no}')['item']


def funds() -> _DataFrame:
    """Return the data available at https://fund.fipiran.ir/mf/list.

    Also see fipiran.data_service.mutual_fund_list function.
    """
    df = _DataFrame(_api('fund/fundcompare')['items'], copy=False).astype(
        {
            'regNo': 'int64',
            'name': 'string',
            'manager': 'string',
            'auditor': 'string',
            'custodian': 'string',
            'guarantor': 'string',
            'date': 'datetime64',
            'typeOfInvest': 'category',
        },
        copy=False,
    )
    return df


def average_returns() -> _DataFrame:
    return _read_html(_fipiran('Fund/MFBazdehAVG'))[0]


def map_data() -> _DataFrame:
    return _DataFrame(_api('fund/treemap')['items'], copy=False).astype(
        {
            'regNo': 'int64',
            'name': 'string',
            'typeOfInvest': 'category',
            'initiationDate': 'datetime64',
            'date': 'datetime64',
            'manager': 'string',
            'auditor': 'string',
            'custodian': 'string',
            'guarantor': 'string',
        },
        copy=False,
    )


def dependency_graph_data() -> _DataFrame:
    return _DataFrame(_api('fund/dependencygraph')['items'], copy=False).astype(
        {
            'regNo': 'int64',
            'name': 'string',
            'date': 'datetime64',
            'manager': 'string',
            'managerCode': 'string',
            'guarantor': 'string',
            'guarantorCode': 'string',
            # 'rankLastUpdate': 'datetime64',
            'typeOfInvest': 'category',
            'initiationDate': 'datetime64',
        },
        copy=False,
    )



from typing import TypedDict as _TypedDict

from . import _fipiran, _DataFrame, _read_html, _to_datetime, _jdatetime


_jstrptime = _jdatetime.strptime


def auto_complete_fund(
    id_: str,
) -> list[_TypedDict('FundAutoComplete', {'RegNo': int, 'Name': str})]:
    return _fipiran('DataService/AutoCompletefund', (('id', id_),), True)


def auto_complete_index(
    id_: str,
) -> list[_TypedDict('IndexAutoComplete', {'LVal30': str, 'InstrumentID': str})]:
    return _fipiran('DataService/AutoCompleteindex', (('id', id_),), True)


def export_index(
    lval30: str, start_date: str | int, end_date: str | int, instrument_id: str = None
) -> _DataFrame:
    """Return history of requested index.

    Use the `auto_complete_index` function to retrieve lval30 and instrument_id
        of the desired index.
    `instrument_id` is optional and if left out then the first result of
        `auto_complete_symbol` will be used.
    Date parameters should be SH dates in YYYYMMDD format e.g.:
        '13940101'
    """
    if instrument_id is None:
        d = auto_complete_index(lval30)[0]
        lval30 = d['LVal30']
        instrument_id = d['InstrumentID']
    xls = _fipiran(
        'DataService/ExportIndex',
        (
            ('indexpara', lval30),
            ('inscodeindex', instrument_id),
            ('indexStart', start_date),
            ('indexEnd', end_date),
        ),
    )
    df = _read_html(xls)[0]
    df['dateissue'] = df['dateissue'].apply(str).apply(_jstrptime, args=('%Y%m%d',))
    return df


def auto_complete_symbol(
    id_: str,
) -> list[_TypedDict('SymbolAutoComplete', {'LVal18AFC': str, 'InstrumentID': str})]:
    return _fipiran('DataService/AutoCompletesymbol', (('id', id_),), True)


def export_symbol(
    lval18afc: str,
    start_date: str | int,
    end_date: str | int,
    instrument_id: str = None,
) -> _DataFrame:
    """Return history of requested index.

    Use the `auto_complete_symbol` function to retrieve `lval18afc` and
        `instrument_id` of the desired symbol.
    `instrument_id` is optional and if left out then the first result of
        `auto_complete_symbol` will be used.

    Date parameters should be SH dates in YYYYMMDD format e.g.:
        '13940101'
    """
    if instrument_id is None:
        d = auto_complete_symbol(lval18afc)[0]
        lval18afc = d['LVal18AFC']
        instrument_id = d['InstrumentID']
    xls = _fipiran(
        'DataService/Exportsymbol',
        (
            ('symboldatapara', lval18afc),
            ('inscodesymbol', instrument_id),
            ('symbolStart', start_date),
            ('symbolEnd', end_date),
        ),
    )
    df = _read_html(xls)[0]
    df['PDate'] = df['PDate'].apply(str).apply(_jstrptime, args=('%Y%m%d',))
    df['GDate'] = _to_datetime(df['GDate'], format='%Y%m%d')
    return df


def balance_sheet(
    l18: str,
    year: str | int,
) -> _DataFrame:
    """Return balance sheet for the requested symbol name as DataFrame.

    https://www.fipiran.ir/DataService/BSIndex
    """
    xls = _fipiran(
        'DataService/ExportBS',
        (
            ('symbolparaBS', l18),
            ('year', year),
        ),
    )
    df = _read_html(xls)[0]
    jdate_cols = ['PublishDate', 'FinanceYear']
    df[jdate_cols] = (
        df[jdate_cols].applymap(str).applymap(_jstrptime, format='%Y/%m/%d')
    )
    return df


def profit_loss(
    l18: str,
    year: str | int,
) -> _DataFrame:
    """Return profit and loss statements for the requested symbol and year.

    https://www.fipiran.ir/DataService/ISIndex
    """
    xls = _fipiran(
        'DataService/ExportIS',
        (
            ('symbolparaIS', l18),
            ('year', year),
        ),
    )
    df = _read_html(xls)[0]
    jdate_cols = ['publishDate', 'FinanceYear']
    df[jdate_cols] = (
        df[jdate_cols].applymap(str).applymap(_jstrptime, format='%Y/%m/%d')
    )
    return df


def financial_ratios(
    l18: str,
    year: str | int,
) -> _DataFrame:
    """Return financial ratios for the requested symbol and year.

    https://www.fipiran.ir/DataService/RatioIndex
    """
    xls = _fipiran(
        'DataService/ExportRatio',
        (
            ('symbolparaR', l18),
            ('year', year),
        ),
    )
    df = _read_html(xls)[0]
    jdate_cols = ['PublishDate', 'FinancialYear']
    df[jdate_cols] = (
        df[jdate_cols].applymap(str).applymap(_jstrptime, format='%Y/%m/%d')
    )
    return df



__version__ = '0.8.1.dev0'

from requests import get as _get

# noinspection PyUnresolvedReferences
from pandas import (
    DataFrame as _DataFrame,
    read_html as _read_html,
    to_datetime as _to_datetime,
)

# noinspection PyUnresolvedReferences
from jdatetime import datetime as _jdatetime


_FIPIRAN = 'https://www.fipiran.ir/'
_YK = ''.maketrans('يك', 'یک')
_API = 'https://fund.fipiran.ir/api/v1/'


def _api(path) -> dict | list:
    return _get(_API + path).json()


def _fipiran(path: str, data=None, json_resp=False) -> str | dict | list:
    resp = _get(f'{_FIPIRAN}{path}', data)
    if json_resp is True:
        return resp.json()
    return resp.content.decode().translate(_YK)
